import { useRef, useEffect, useCallback, useState } from 'react';
import { useQueryClient, useQuery, useMutation } from 'react-query';
import { onSnapshot, getDoc, getDocFromServer, getDocFromCache, namedQuery as namedQuery$1, queryEqual, getDocs, getDocsFromServer, getDocsFromCache, addDoc, setDoc } from 'firebase/firestore';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function __generator(thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
}

/*
 * Copyright (c) 2016-present Invertase Limited & Contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this library except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
function usePrevious(value) {
    var ref = useRef();
    useEffect(function () {
        ref.current = value;
    });
    return ref.current;
}

/*
 * Copyright (c) 2016-present Invertase Limited & Contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this library except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
function getClientKey(key) {
    if (typeof key === "string") {
        return key;
    }
    return key.join("");
}

/*
 * Copyright (c) 2016-present Invertase Limited & Contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this library except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
function useSubscription(enabled, ref, includeMetadataChanges, onSnapshotEvent) {
    var previousRef = usePrevious(ref);
    var isEqual = !!previousRef && ref.id === previousRef.id;
    var unsubscribe = useRef();
    useEffect(function () {
        if (enabled && !isEqual) {
            unsubscribe.current = onSnapshot(ref, {
                includeMetadataChanges: includeMetadataChanges,
            }, onSnapshotEvent);
        }
    }, [enabled, isEqual, ref, includeMetadataChanges, onSnapshotEvent]);
    // Unsubscribes the ref subscription when the ref changes.
    useEffect(function () {
        if (!isEqual && !!previousRef) {
            return function () {
                var _a;
                (_a = unsubscribe.current) === null || _a === void 0 ? void 0 : _a.call(unsubscribe);
            };
        }
    }, [isEqual, previousRef]);
    // Unsubscribe when the hook is no longer in use.
    useEffect(function () {
        return function () {
            var _a;
            (_a = unsubscribe.current) === null || _a === void 0 ? void 0 : _a.call(unsubscribe);
        };
    }, []);
}
function getSnapshot$1(ref, source) {
    return __awaiter(this, void 0, void 0, function () {
        var snapshot;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!(source === "cache")) return [3 /*break*/, 2];
                    return [4 /*yield*/, getDocFromCache(ref)];
                case 1:
                    snapshot = _a.sent();
                    return [3 /*break*/, 6];
                case 2:
                    if (!(source === "server")) return [3 /*break*/, 4];
                    return [4 /*yield*/, getDocFromServer(ref)];
                case 3:
                    snapshot = _a.sent();
                    return [3 /*break*/, 6];
                case 4: return [4 /*yield*/, getDoc(ref)];
                case 5:
                    snapshot = _a.sent();
                    _a.label = 6;
                case 6: return [2 /*return*/, snapshot];
            }
        });
    });
}
function useFirestoreDocument(key, ref, options, useQueryOptions) {
    var _a, _b, _c;
    var client = useQueryClient();
    var subscribe = (_a = options === null || options === void 0 ? void 0 : options.subscribe) !== null && _a !== void 0 ? _a : false;
    var enabled = (_b = useQueryOptions === null || useQueryOptions === void 0 ? void 0 : useQueryOptions.enabled) !== null && _b !== void 0 ? _b : true;
    var includeMetadataChanges = (options === null || options === void 0 ? void 0 : options.subscribe)
        ? (_c = options === null || options === void 0 ? void 0 : options.includeMetadataChanges) !== null && _c !== void 0 ? _c : undefined
        : undefined;
    var compareKey = getClientKey(key);
    var onSnapshotEvent = useCallback(function (snapshot) {
        client.setQueryData(key, snapshot);
    }, [compareKey]);
    useSubscription(subscribe && enabled, ref, includeMetadataChanges, onSnapshotEvent);
    return useQuery(key, function () { return getSnapshot$1(ref, options === null || options === void 0 ? void 0 : options.source); }, useQueryOptions);
}
function useFirestoreDocumentData(key, ref, options, useQueryOptions) {
    var _this = this;
    var _a, _b, _c;
    var client = useQueryClient();
    var subscribe = (_a = options === null || options === void 0 ? void 0 : options.subscribe) !== null && _a !== void 0 ? _a : false;
    var enabled = (_b = useQueryOptions === null || useQueryOptions === void 0 ? void 0 : useQueryOptions.enabled) !== null && _b !== void 0 ? _b : true;
    var includeMetadataChanges = (options === null || options === void 0 ? void 0 : options.subscribe)
        ? (_c = options === null || options === void 0 ? void 0 : options.includeMetadataChanges) !== null && _c !== void 0 ? _c : undefined
        : undefined;
    var compareKey = getClientKey(key);
    var onSnapshotEvent = useCallback(function (snapshot) {
        client.setQueryData(key, snapshot.data({
            serverTimestamps: options === null || options === void 0 ? void 0 : options.serverTimestamps,
        }));
    }, [compareKey]);
    useSubscription(subscribe && enabled, ref, includeMetadataChanges, onSnapshotEvent);
    return useQuery(key, function () { return __awaiter(_this, void 0, void 0, function () {
        var snapshot;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, getSnapshot$1(ref, options === null || options === void 0 ? void 0 : options.source)];
                case 1:
                    snapshot = _a.sent();
                    return [2 /*return*/, snapshot.data({
                            serverTimestamps: options === null || options === void 0 ? void 0 : options.serverTimestamps,
                        })];
            }
        });
    }); }, useQueryOptions);
}

/*
 * Copyright (c) 2016-present Invertase Limited & Contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this library except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
var namedQueryCache = {};
function isNamedQuery(query) {
    return typeof query === "function";
}
function useQueryResolver(enabled, query, includeMetadataChanges, onSnapshotEvent) {
    // Separate the query and named query types.
    var queryFn;
    var namedQueryFn;
    // If the user passed a named query function.
    if (isNamedQuery(query)) {
        // Check whether result is a promise or query.
        if (typeof query === "function") {
            namedQueryFn = query;
        }
        else {
            queryFn = query;
        }
    }
    else {
        queryFn = query;
    }
    var previousQuery = usePrevious(queryFn);
    var isEqual = !!previousQuery && queryEqual(previousQuery, queryFn);
    var _a = useState(null), resolvedQuery = _a[0], setResolvedQuery = _a[1];
    var unsubscribe = useRef();
    // Anytime the query changes, update the resolved query.
    useEffect(function () {
        if (!isEqual && !!queryFn) {
            setResolvedQuery(queryFn);
        }
    }, [isEqual, queryFn]);
    // If a named query is provided, resolve it.
    useEffect(function () {
        if (!resolvedQuery && !!namedQueryFn) {
            namedQueryFn().then(setResolvedQuery);
        }
    }, [resolvedQuery, namedQueryFn]);
    // Subscribes to the resolved query.
    useEffect(function () {
        if (enabled && resolvedQuery) {
            unsubscribe.current = onSnapshot(resolvedQuery, {
                includeMetadataChanges: includeMetadataChanges,
            }, onSnapshotEvent);
        }
    }, [enabled, resolvedQuery, includeMetadataChanges, onSnapshotEvent]);
    // Unsubscribes the query subscription when the query changes.
    useEffect(function () {
        if (!isEqual && !!previousQuery) {
            return function () {
                var _a;
                (_a = unsubscribe.current) === null || _a === void 0 ? void 0 : _a.call(unsubscribe);
            };
        }
    }, [unsubscribe, isEqual, previousQuery]);
    useEffect(function () {
        return function () {
            var _a;
            (_a = unsubscribe.current) === null || _a === void 0 ? void 0 : _a.call(unsubscribe);
        };
    }, []);
    return resolvedQuery;
}
function getSnapshot(query, source) {
    return __awaiter(this, void 0, void 0, function () {
        var snapshot;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!(source === "cache")) return [3 /*break*/, 2];
                    return [4 /*yield*/, getDocsFromCache(query)];
                case 1:
                    snapshot = _a.sent();
                    return [3 /*break*/, 6];
                case 2:
                    if (!(source === "server")) return [3 /*break*/, 4];
                    return [4 /*yield*/, getDocsFromServer(query)];
                case 3:
                    snapshot = _a.sent();
                    return [3 /*break*/, 6];
                case 4: return [4 /*yield*/, getDocs(query)];
                case 5:
                    snapshot = _a.sent();
                    _a.label = 6;
                case 6: return [2 /*return*/, snapshot];
            }
        });
    });
}
function namedQuery(firestore, name) {
    var key = firestore.app.name + ":" + name;
    if (namedQueryCache[key]) {
        return namedQueryCache[key];
    }
    return function () {
        return namedQuery$1(firestore, name).then(function (query) {
            if (query) {
                namedQueryCache[key] = query;
                return query;
            }
            return null;
        });
    };
}
function useFirestoreQuery(key, query, options, useQueryOptions) {
    var _a, _b, _c, _d;
    var client = useQueryClient();
    var subscribe = (_a = options === null || options === void 0 ? void 0 : options.subscribe) !== null && _a !== void 0 ? _a : false;
    var enabled = (_b = useQueryOptions === null || useQueryOptions === void 0 ? void 0 : useQueryOptions.enabled) !== null && _b !== void 0 ? _b : true;
    var includeMetadataChanges = (options === null || options === void 0 ? void 0 : options.subscribe)
        ? (_c = options === null || options === void 0 ? void 0 : options.includeMetadataChanges) !== null && _c !== void 0 ? _c : undefined
        : undefined;
    var compareKey = getClientKey(key);
    var onSnapshotEvent = useCallback(function (snapshot) {
        client.setQueryData(key, snapshot);
    }, [compareKey]);
    var resolvedQuery = useQueryResolver(subscribe && enabled, query, includeMetadataChanges, onSnapshotEvent);
    return useQuery(key, function () { return getSnapshot(resolvedQuery, options === null || options === void 0 ? void 0 : options.source); }, __assign(__assign({}, (useQueryOptions || {})), { 
        // If there is a resolved query, use the users option (or default), otherwise it is false.
        enabled: !!resolvedQuery ? (_d = useQueryOptions === null || useQueryOptions === void 0 ? void 0 : useQueryOptions.enabled) !== null && _d !== void 0 ? _d : undefined : false }));
}
function useFirestoreQueryData(key, query, options, useQueryOptions) {
    var _this = this;
    var _a, _b, _c, _d;
    var client = useQueryClient();
    var subscribe = (_a = options === null || options === void 0 ? void 0 : options.subscribe) !== null && _a !== void 0 ? _a : false;
    var enabled = (_b = useQueryOptions === null || useQueryOptions === void 0 ? void 0 : useQueryOptions.enabled) !== null && _b !== void 0 ? _b : true;
    var includeMetadataChanges = (options === null || options === void 0 ? void 0 : options.subscribe)
        ? (_c = options === null || options === void 0 ? void 0 : options.includeMetadataChanges) !== null && _c !== void 0 ? _c : undefined
        : undefined;
    var compareKey = getClientKey(key);
    var onSnapshotEvent = useCallback(function (snapshot) {
        client.setQueryData(key, snapshot.docs.map(function (doc) {
            return doc.data({
                serverTimestamps: options === null || options === void 0 ? void 0 : options.serverTimestamps,
            });
        }));
    }, [compareKey]);
    var resolvedQuery = useQueryResolver(subscribe && enabled, query, includeMetadataChanges, onSnapshotEvent);
    return useQuery(key, function () { return __awaiter(_this, void 0, void 0, function () {
        var snapshot;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, getSnapshot(resolvedQuery, options === null || options === void 0 ? void 0 : options.source)];
                case 1:
                    snapshot = _a.sent();
                    return [2 /*return*/, snapshot.docs.map(function (doc) {
                            return doc.data({
                                serverTimestamps: options === null || options === void 0 ? void 0 : options.serverTimestamps,
                            });
                        })];
            }
        });
    }); }, __assign(__assign({}, (useQueryOptions || {})), { 
        // If there is a resolved query, use the users option (or default), otherwise it is false.
        enabled: !!resolvedQuery ? (_d = useQueryOptions === null || useQueryOptions === void 0 ? void 0 : useQueryOptions.enabled) !== null && _d !== void 0 ? _d : undefined : false }));
}

/*
 * Copyright (c) 2016-present Invertase Limited & Contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this library except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
function useFirestoreCollectionMutation(ref, useMutationOptions) {
    return useMutation(function (data) {
        return addDoc(ref, data);
    }, useMutationOptions);
}
function useFirestoreDocumentMutation(ref, options, useMutationOptions) {
    return useMutation(function (data) {
        if (!!options) {
            return setDoc(ref, data, options);
        }
        return setDoc(ref, data);
    }, useMutationOptions);
}

export { namedQuery, useFirestoreCollectionMutation, useFirestoreDocument, useFirestoreDocumentData, useFirestoreDocumentMutation, useFirestoreQuery, useFirestoreQueryData };
